"""Docker functions to get info about containers"""


from json import loads as json_loads
import subprocess


running_cts = 0
cts_info = dict()


def check_cts_are_running(project_name: str, config: str = None):
    """Throws an error if cts are not running"""

    global cts_info

    running_cts, cts_info = get_running_containers(project_name, config)
    if running_cts is 0:
        raise SystemError('Have you started your server with the start action ?')

    return (running_cts, cts_info)


def container_running(name: str):
    """Returns True if the container is running else False"""

    cmd = ['docker', 'inspect', '-f', '{{.State.Running}}', name]
    try:
        result = subprocess.check_output(cmd, stderr=subprocess.STDOUT).splitlines()[0]
        return False if result.decode() == 'false' else True
    except subprocess.CalledProcessError as e:
        return False


def get_ct_item(compose_name: str, item_name: str):
    """Get a value from a container, such as name or IP"""
    for ct_id, ct_data in cts_info.items():
        if ct_data['compose_name'] == compose_name:
            return ct_data[item_name]

    return ''


def get_ct_name(ct: str):
    ct_name = get_ct_item(ct, 'name')
    if ct_name == '':
        raise Exception('{} does not seem to be started ...'.format(ct))

    return ct_name


def get_running_containers(project_name: str, config: str = None):
    """Get a list of IDs of running containers for the current stakkr instance"""

    global cts_info

    config_params = []
    if config is not None:
        config_params = ['-c', config]

    running_cts = 0
    cts_id = subprocess.check_output(['stakkr-compose'] + config_params + ['ps', '-q'])
    cts_info = dict()
    for ct_id in cts_id.splitlines():
        ct_id = ct_id.decode()
        cts_info[ct_id] = _extract_container_info(project_name, ct_id)

        if cts_info[ct_id]['running'] is True:
            running_cts += 1

    return (running_cts, cts_info)


def _extract_container_info(project_name: str, ct_id: str):
    """Get a hash of info about a container : name, ports, image, ip ..."""

    try:
        result = subprocess.check_output(['docker', 'inspect', ct_id], stderr=subprocess.STDOUT)
        ct_data = json_loads(result.decode().rstrip('\n'))

        ct_info = {
            'name': ct_data[0]['Name'].lstrip('/'),
            'compose_name': ct_data[0]['Config']['Labels']['com.docker.compose.service'],
            'ports': ct_data[0]['Config']['ExposedPorts'].keys() if 'ExposedPorts' in ct_data[0]['Config'] else [],
            'image': ct_data[0]['Config']['Image'],
            'ip': _get_ip_from_networks(project_name, ct_data[0]['NetworkSettings']['Networks']),
            'running': ct_data[0]['State']['Running'],
        }

        return ct_info
    except subprocess.CalledProcessError as e:
        return None


def _get_ip_from_networks(project_name: str, networks: list):
    """Get a list of IPs for a network"""

    network_settings = {}
    if '{}_stakkr'.format(project_name) in networks:
        network_settings = networks['{}_stakkr'.format(project_name)]

    return network_settings['IPAddress'] if 'IPAddress' in network_settings else ''


def create_network(network: str):
    """Create a Network"""

    if _network_exists(network) is True:
        return False

    cmd = ['docker', 'network', 'create', '--driver', 'bridge', network]
    subprocess.check_output(cmd, stderr=subprocess.STDOUT)

    return True


def add_container_to_network(container: str, network: str):
    """Attach a container to a network"""

    if _container_in_network(container, network) is True:
        return False

    cmd = ['docker', 'network', 'connect', network, container]
    subprocess.check_output(cmd, stderr=subprocess.STDOUT)

    return True


def _container_in_network(container: str, expected_network: str):
    """Returns True if a container is in a network else false. Used by add_container_to_network"""

    try:
        cmd = ['docker', 'inspect', container]
        result = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
    except Exception:
        raise SystemError('Container {} does not seem to exist'.format(container))

    ct_data = json_loads(result.decode().rstrip('\n'))
    for connected_network in ct_data[0]['NetworkSettings']['Networks'].keys():
        if connected_network == expected_network:
            return True

    return False


def _network_exists(network: str):
    """Returns True if a network exists. Used by create_network"""

    cmd = ['docker', 'network', 'ls', '--filter', 'name={}'.format(network)]
    result = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
    networks_list = result.decode().splitlines()
    if len(networks_list) > 1:
        return True

    return False



"""Gives useful information about the current virtualenv, files locations if
stakkr is installed as a package or directly cloned"""


import os

from distutils.sysconfig import get_python_lib


def get_venv_basedir():
    """Returns the base directory of the virtualenv, useful to read configuration and plugins"""

    try:
        import virtualenv
        raise EnvironmentError("You must be in a virtual environment")
    except Exception:
        return os.path.abspath(get_python_lib() + '/../../../../')


def get_dir(dirname: str):
    """Detects if stakkr is a package or a clone and gives the right path for a directory"""

    staticdir = os.path.dirname(os.path.realpath(__file__)) + '/' + dirname
    if os.path.isdir(staticdir) is True:
        return staticdir

    return get_python_lib() + '/stakkr/' + dirname


def get_file(dirname: str, filename: str):
    """Detects if stakkr is a package or a clone and gives the right path for a file"""

    dir_path = get_dir(dirname)

    return dir_path + '/' + filename.lstrip('/')



#!/usr/bin/env python3
"""
Clean unused containers, images, volumes and networks
That saves a lot of space ... should be removed later by "docker ***** prune
"""

import click
import sys

from subprocess import Popen, DEVNULL, PIPE, STDOUT


@click.command(
    help="Clean Docker containers, images, volumes and networks that are not in use",
    name="docker-clean")
@click.option('--force', '-f', help="Do it", is_flag=True)
@click.option('--verbose', '-v', help="Display more information about what is removed", is_flag=True)
def clean(force: bool, verbose: bool):
    print(click.style('Clean Docker stopped containers, images, volumes and networks', fg='green'))

    remove_containers(force, verbose)
    print()
    remove_images(force, verbose)
    print()
    remove_volumes(force, verbose)
    print()
    remove_networks(force, verbose)

    if force is False:
        print(click.style("\n--force is not set so I won't do anything\n", fg='red'))

    print(click.style('Done', fg='green'))


def remove_containers(force: bool, verbose: bool):
    containers = _exec_cmd(['docker', 'ps', '--no-trunc', '-a', '-q', '-f', 'status=exited'])

    if len(containers) == 0:
        print('No exited container to remove')
        return

    print('Removing {} exited container(s)'.format(len(containers)))

    for container in containers:
        _display_entry_info('container', container.decode(), verbose)
        _remove_entry('container', container.decode(), force)



def remove_images(force: bool, verbose: bool):
    images = _exec_cmd(['docker', 'image', 'ls'])

    if len(images) == 0:
        print('No image to remove')
        return

    print('Removing {} image(s)'.format(len(images)))
    if force is True:
        _prune_images()


def remove_networks(force: bool, verbose: bool):
    networks = _exec_cmd(['docker', 'network', 'ls', '--no-trunc', '-q', '--filter', 'type=custom'])

    if len(networks) == 0:
        print('No network to remove')
        return

    print('Removing {} exited networks(s)'.format(len(networks)))

    for network in networks:
        _display_entry_info('network', network.decode(), verbose)
        _remove_entry('network', network.decode(), force)


def remove_volumes(force: bool, verbose: bool):
    volumes = _exec_cmd(['docker', 'volume', 'ls', '-q', '-f', 'dangling=true'])

    if len(volumes) == 0:
        print('No volume to remove')
        return

    print('Removing {} exited volumes(s)'.format(len(volumes)))

    for volume in volumes:
        if verbose is True:
            print('  Removing volume {}'.format(volume.decode()))

        _remove_entry('volume', volume.decode(), force)


def _display_entry_info(entry_type: str, entry: str, verbose: bool):
    if verbose is False:
        return

    base_cmd = ['docker']
    if entry_type != 'container':
        base_cmd += [entry_type]

    info = _exec_cmd(base_cmd + ['inspect', '--format={{.Name}}', entry])[0]
    print('  Removing {} {}'.format(entry_type, info.decode()))


def _exec_cmd(cmd: list):
    return Popen(cmd, stdout=PIPE, stderr=STDOUT).communicate()[0].splitlines()


def _prune_images():
    try:
        _exec_cmd(['docker', 'image', 'prune', '--all', '--force'])
    except Exception as e:
        print(click.style('Error removing images'), fg='red')


def _remove_entry(entry_type: str, entry: str, force: bool):
    if force is False:
        return

    try:
        base_cmd = ['docker']
        if entry_type != 'container':
            base_cmd += [entry_type]
        _exec_cmd(base_cmd + ['rm', entry])
    except Exception as e:
        output = e.output.decode()
        print(click.style('Error removing a {}: {}'.format(entry_type, output), fg='red'))


def main():
    try:
        clean()
    except Exception as e:
        msg = click.style(r""" ______ _____  _____   ____  _____
|  ____|  __ \|  __ \ / __ \|  __ \
| |__  | |__) | |__) | |  | | |__) |
|  __| |  _  /|  _  /| |  | |  _  /
| |____| | \ \| | \ \| |__| | | \ \
|______|_|  \_\_|  \_\\____/|_|  \_\

""", fg='yellow')
        msg += click.style('{}'.format(e), fg='red')

        print(msg)
        print()
        # raise e
        sys.exit(1)


if __name__ == '__main__':
    main()



import os
import sys

from distutils.core import setup
from stakkr.setup import StakkrPostInstall


here = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, here)


setup(
    name='stakkr',
    version='3.2.3',
    description='A configurable stack based on docker to run any combination of services (PHP, MySQL, Apache, Nginx, Mongo, etc..)',
    long_description=""".. image:: https://raw.githubusercontent.com/edyan/stakkr/master/docs/stakkr-logo.png
    :width: 200px
    :align: center

Overview
==================================
.. image:: https://scrutinizer-ci.com/g/edyan/stakkr/badges/quality-score.png?b=master
.. image:: https://scrutinizer-ci.com/g/edyan/stakkr/badges/build.png?b=master


Stakkr is a a docker recompose tool that uses docker compose to easily
create / maintain a stack of services, for example for web development.

Via a configuration file you can setup the required services and
let stakkr link and start everything for you.

It works only in CLI.

Full Documentation: http://stakkr.readthedocs.org


Versions
=============
.. IMPORTANT::
   Stakkr has known 3 versions and 3 names :
   - 1.x : it was called ``docker-lamp`` and was made to setup an Apache + PHP + MySQL Environment
   - 2.x : it was called ``marina`` (nice name!) and was made to have a stack build around various
   services.
   - 3.x : current version (and probably future ones) , now ``stakkr``. Why changing its name again ?
   Because marina is used by various people and therefore our tool could add more confusion.



What does that do exactly ?
==============================
If you have heard of Docker, you know that when you need to build a full environment
with multiple services that are linked, you either have to do everything manually or
use `docker-compose`. The second solution is the best _but_ it implies that you need, for each
environment, to change your parameters, choose your images, learn the `docker-compose` command
line tool, etc ... In brief, it's not very flexible and hard to learn.

Stakkr will help you, via a very simple configuration file and a predefined list of services
(that can be extended by plugins) to build a complete environment. Plus, to control it in command line.
It makes use of docker easy.

Last, but not the least, it's highly configurable and each service mounts a volume to have a persistence
of data. You can even, if you want, add more directives on some services (change the `php.ini` for
example and chose your versions (PHP 5.3 or 5.6 or 7.1 or anything else).


Examples
==============
You can combine services as you want to have :

- A **Dev LAMP stack** (Apache + MySQL 5.7 + PHP 7.1 with xdebug and xhprof) ...
  and if suddenly you want to test your code with PHP 7.1,
  change it in `conf/compose.ini`, restart, it's done !
- Or Apache 2.4 + PHP 5.6 + MongoDB for a **production environment**
- Or **only Maildev**
- Or **only PHP 5.4 + ElasticSearch**
- etc...""",
    url='http://github.com/edyan/stakkr',
    author='Emmanuel Dyan',
    author_email='emmanueldyan@gmail.com',
    license='Apache 2.0',
    packages=['stakkr'],
    py_modules=['stakkr'],
    entry_points='''[console_scripts]
stakkr=stakkr.cli:main
docker-clean=stakkr.docker_clean:main
stakkr-compose=stakkr.stakkr_compose:cli''',
    include_package_data=True,
    install_requires=[
        'clint',
        'click', 'click-plugins',
        'docker-compose',
        'configobj',
        'requests>=2.11.0,<2.12'
        ],
    cmdclass={'install': StakkrPostInstall},
    classifiers=[
      'Development Status :: 4 - Beta',
      'Environment :: Console',
      'Intended Audience :: Developers',
      'License :: OSI Approved :: Apple Public Source License',
      'Programming Language :: Python :: 3',
      'Programming Language :: Python :: 3.5',
    ],
    keywords='docker,docker-compose,python,stack,development,lamp',
)
